# CONTENT:
# - classes (operators, property groups etc.) 
#   - main addon program
#   - gui 
bl_info = {
    "name": "Demo s loggerem",
    "description": "Demonstrační modul s loggerem 2.",
    "author": "Jiří Chludil",
    "version": (1, 0, 0),
    "blender": (2, 93, 5),
    "category": "Add Mesh"
}

import bpy, os, sys
from random import uniform, randint
from bpy.props import *
import random
import requests
import json
import time
from bpy_extras import object_utils

# -----------------------------------------------------------------------

class Info(bpy.types.PropertyGroup):
    """
    Třída pro podporu infomačního okna modulu 

    Attributes:
    --------------
    line : string
        řádek infomačního panelu
    """

    line : bpy.props.StringProperty(
          name = ""
    )
    
# -----------------------------------------------------------------------

class InfoGroup(bpy.types.PropertyGroup):
    """ Třída používající Blender Property Group pro správu informačního okna ."""
    coll : bpy.props.CollectionProperty(type=Info)
    index : bpy.props.IntProperty() 

# -----------------------------------------------------------------------

class InfoList(bpy.types.UIList):
    """ Třída používající Blender UIList definující vzhled informačního okna """

    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):

        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            layout.prop(item,"line", emboss=False)                  
        elif self.layout_type in { 'GRID'}:
            layout.alignment = 'CENTER'
            layout.label(text="", icon_value=icon)
            
 # -----------------------------------------------------------------------           

class AddInfo(bpy.types.Operator):
    
    bl_idname = "info.add"
    bl_label = "Vlož záznam do infomačního okna"
   
    info : bpy.props.StringProperty(default="OBJECT")
   
    def execute(self, context):
        item = context.scene.prop_group_info.coll.add()
        item.line = self.info
        context.scene.prop_group_info.coll.move(len(context.scene.prop_group_info.coll.items())-1,0)
        return {'FINISHED'}

 # -----------------------------------------------------------------------     
    
class ClearInfoList(bpy.types.Operator):
    """ Třída používající Blender`s Operator k výmazu obsahu informačního okna."""

    bl_idname = "info.clear"
    bl_label = "Vymaž infomační okno"
    
    def execute(self, context):
        context.scene.prop_group_info.coll.clear()
        return {'FINISHED'}    

# -----------------------------------------------------------------------

class Akce(bpy.types.Operator):
    bl_idname = "demo.akce"
    bl_label = "Akce"
    
    def execute(self, context):
        bpy.ops.info.add(info="Akce spustena")   
        return {'FINISHED'}

# -----------------------------------------------------------------------

class DemoPanel(bpy.types.Panel):
    """Class creating a Panel in Tool properties."""

    bl_label = "Demo"
    bl_idname = "demo"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Tool"
    bl_context = "objectmode"

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        house = scene.house
    
        row = layout.row(align=True) 
        row.prop(house, "size")
        row = layout.row(align=True) 
        row.prop(house, "roof")
        row = layout.row(align=True) 
        row.operator("object.house2")
        row = layout.row(align=True) 
        
        # Informační okno
        layout.template_list("InfoList", "", scene.prop_group_info, "coll", \
            scene.prop_group_info, "index", rows=5, maxrows=5, columns=3, type='DEFAULT')
        row = layout.row(align=True) 
        row.operator("info.clear")    
       
class HouseData(bpy.types.PropertyGroup):
 
    
    size : bpy.props.FloatVectorProperty(
        name = 'Size',
        description = 'Size of base house',
        subtype = 'XYZ',
        default=(1.0, 1.0, 1.0),
    )
    
    roof : bpy.props.FloatProperty(
       name = 'Roof',
       description = 'Height of roof',
       default = 1,
       min = 0,
       max = 2,
       step = 25,
    )
    
          
class GenHouse(bpy.types.Operator):  
    bl_idname = "object.house2"
    bl_label = "Generate House"
    bl_options = {'REGISTER', 'UNDO'}
  
    def execute(self, context):
        bpy.ops.info.add(info="Startuji")   
        verts, edges, faces = self.add_house(context.scene.house.size, context.scene.house.roof)
        bpy.ops.info.add(info="Mesh vygenerovan")   
        obj = self.create_mesh_object(context, verts, edges, faces, "House") 
        bpy.ops.info.add(info="Domecek postaven")       
        return {'FINISHED'}
        
    def add_house(self, size = (1.0 ,1.0 ,1.0 ), roof = 1):
        verts = []
        edges = []
        faces = []
        size_x = size[0] / 2.0
        size_y = size[1] / 2.0
        size_z = size[2] / 2.0
        verts.append((-size_x, -size_y, -size_z))
        verts.append(( size_x, -size_y, -size_z))
        verts.append(( size_x, size_y, -size_z))
        verts.append((-size_x, size_y, -size_z))
        verts.append((-size_x, -size_y, size_z))
        verts.append(( size_x, -size_y, size_z))
        verts.append(( size_x, size_y, size_z))
        verts.append((-size_x, size_y, size_z))
        verts.append((-size_x, 0, size_z+roof))
        verts.append(( size_x, 0, size_z+roof))
        faces.append([0,3,2,1])
        faces.append([0,1,5,4])
        faces.append([1,2,6,5])
        faces.append([2,3,7,6])
        faces.append([3,0,4,7])
        faces.append([4,5,9,8])
        faces.append([6,7,8,9])
        faces.append([5,6,9])
        faces.append([4,8,7])
        return verts, edges, faces

    def create_mesh_object(self, context, verts, edges, faces, name):
        mesh = bpy.data.meshes.new(name)
        mesh.from_pydata(verts, edges, faces)
        mesh.update()
        
        return object_utils.object_data_add(context, mesh, operator=None)      
    
        
#def menu_func(self, context):
#    return
    
        
classes = [DemoPanel, ClearInfoList, Akce, Info, InfoGroup, InfoList, AddInfo, HouseData, GenHouse]

def menu_func(self, context):
    self.layout.operator(DemoPanel.bl_idname)

def register():
    from bpy.utils import register_class
    for cls in classes:
       register_class(cls)
    bpy.types.Scene.house = PointerProperty(type=HouseData)
    bpy.types.Scene.prop_group_info = PointerProperty(type=InfoGroup)   
   #bpy.types.VIEW3D_MT_mesh_add.append(menu_func)       

def unregister():
    from bpy.utils import unregister_class
    for cls in reversed(classes):
       unregister_class(cls)
    del bpy.types.Scene.prop_group_info
    del bpy.types.Scene.items    
    #bpy.types.VIEW3D_MT_mesh_add.remove(menu_func)

if __name__ == "__main__":
    #unregister()
    register()
